/* xsoldier, a shoot 'em up game with "not shooting" bonus
 * Copyright (C) 1997 Yuusuke HASHIMOTO <s945750@educ.info.kanagawa-u.ac.jp>
 * Copyright (C) 2002 Oohara Yuuma  <oohara@libra.interq.or.jp>
 *
 * This is a copyleft program.  See the file LICENSE for details.
 */
/* $Id: graphic.c,v 1.10 2002/04/20 01:22:26 oohara Exp $ */
#include <config.h>

#include <stdio.h>
/* isprint */
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

/* Image */
#include "image.h"
#include "graphic.h"
#include "xsoldier.h"
#include "star.h"

#include "extern.h"
#if 0
static GC FontGC;
static GC BackGC;
static GC FillGC;
static XColor black;
static XColor white;

static Display *dpy;
static Colormap cmap;
static Window root;
static Window win;
static Pixmap WorkPixmap;
#endif /* 0 */


int graphic_init(void)
{
  XColor blackTrue;
  XColor whiteTrue;
  XSizeHints sh;

  FieldW  = 500;
  FieldH  = 650;

  dpy = XOpenDisplay(display);
  if (dpy == NULL)
  {
    fprintf(stderr,"graphic_init: [%s] can't open display.\n", display);
    exit(1);
  }

  if (colormap == False)
    cmap = XDefaultColormap(dpy, 0);
  else
    cmap = XCreateColormap(dpy, RootWindow(dpy, 0), DefaultVisual(dpy, 0),
                           AllocNone);

  XAllocNamedColor(dpy, cmap, "black", &black, &blackTrue);
  XAllocNamedColor(dpy, cmap, "white", &white, &whiteTrue);

  root = XCreateSimpleWindow(dpy, RootWindow(dpy,0), 0, 0, FieldW+20,
                             FieldH+20, 0, white.pixel, black.pixel);
  if (colormap == False)
    XSetWindowColormap(dpy, root, cmap);

  XSetWindowColormap(dpy,root,cmap);
  XSelectInput(dpy, root, ExposureMask|EnterWindowMask|LeaveWindowMask|KeyPressMask|KeyReleaseMask);
  XStoreName(dpy, root, "xsoldier");
  sh.flags = (PMaxSize | PMinSize);
  sh.min_width = FieldW + 20;
  sh.min_height = FieldH + 20;
  sh.max_width = FieldW + 20;
  sh.max_height = FieldH + 20;
  XSetWMNormalHints(dpy, root, &sh);

  win = XCreateSimpleWindow(dpy, root, 10, 10, FieldW, FieldH, 1,
                            white.pixel, black.pixel);
  XSelectInput(dpy, win, ExposureMask|EnterWindowMask|KeyPressMask|KeyReleaseMask);

  WorkPixmap = XCreatePixmap(dpy, win, FieldW, FieldH, DefaultDepth(dpy, 0));
  font       = XLoadFont(dpy,"-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1");
  FontGC     = XCreateGC(dpy,root,0,0);
  XSetGraphicsExposures(dpy,FontGC,False);
  XSetForeground(dpy,FontGC,white.pixel);
  XSetFont(dpy,FontGC,font);

  BackGC       = XCreateGC(dpy,WorkPixmap,0,0);
  XSetGraphicsExposures(dpy,BackGC,False);
  XSetForeground(dpy,BackGC,black.pixel);

  FillGC       = XCreateGC(dpy,WorkPixmap,0,0);
  XSetGraphicsExposures(dpy,FillGC,False);
  XSetForeground(dpy,FillGC,white.pixel);

  PlayerImage = ImageInit(PIXMAP "/Player.xpm" GZ,6);
  PShot1Image  = ImageInit(PIXMAP "/PlayerShot1.xpm" GZ,2);
  PShot2Image  = ImageInit(PIXMAP "/PlayerShot2.xpm" GZ,2);
  PShot3Image  = ImageInit(PIXMAP "/PlayerShot3.xpm" GZ,3);

  EShotImage  = ImageInit(PIXMAP "/EnemyShot.xpm" GZ,4);
  ELaserImage  = ImageInit(PIXMAP "/EnemyLaser.xpm" GZ,1);
  EMissileImage  = ImageInit(PIXMAP "/EnemyMiss.xpm" GZ,8);
  EBoundImage = ImageInit(PIXMAP "/EnemyBound.xpm" GZ,8);
  ERingImage = ImageInit(PIXMAP "/EnemyRing.xpm" GZ,4);

  BombImage   = ImageInit(PIXMAP "/ExpSmall.xpm" GZ,5);
  LargeBombImage= ImageInit(PIXMAP "/ExpLarge.xpm" GZ,5);

  Enemy1Image = ImageInit(PIXMAP "/Enemy1.xpm" GZ,8);
  Enemy2Image = ImageInit(PIXMAP "/Enemy2.xpm" GZ,8);
  Enemy3Image = ImageInit(PIXMAP "/Enemy3.xpm" GZ,8);
  Enemy4Image = ImageInit(PIXMAP "/Enemy4.xpm" GZ,8);
  Enemy5Image = ImageInit(PIXMAP "/Enemy5.xpm" GZ,4);
  Enemy6Image = ImageInit(PIXMAP "/Enemy6.xpm" GZ,6);
  Enemy7Image = ImageInit(PIXMAP "/Enemy7.xpm" GZ,1);

  Boss1Image = ImageInit(PIXMAP "/Boss1.xpm" GZ,1);
  Boss2Image = ImageInit(PIXMAP "/Boss2.xpm" GZ,1);
  Boss3Image = ImageInit(PIXMAP "/Boss3.xpm" GZ,1);
  Boss4Image = ImageInit(PIXMAP "/Boss4.xpm" GZ,1);
  Boss5Image = ImageInit(PIXMAP "/Boss5.xpm" GZ,1);
  Boss6Image = ImageInit(PIXMAP "/Boss6.xpm" GZ,2);
  Boss7Image = ImageInit(PIXMAP "/Boss7.xpm" GZ,1);

  ItemImage = ImageInit(PIXMAP "/Item.xpm" GZ,4);
    
  /* initialize stars */
  InitStarModule(FieldW,FieldH);
  StarPtn1 = CreateStar(PIXMAP "/Star1.xpm" GZ,4,5,5);
  StarPtn2 = CreateStar(PIXMAP "/Star2.xpm" GZ,4,10,10);



  return 0;
}

int clear_window(void)
{
  XFillRectangle(dpy, WorkPixmap, BackGC, 0, 0, FieldW, FieldH);
  return 0;
}


int redraw_window(void)
{
  XCopyArea(dpy,WorkPixmap,win,BackGC,0,0,FieldW,FieldH,0,0);
  XFlush(dpy);
  XSync(dpy,False);
  return 0;
}

int graphic_finish(void)
{
  FreeImages(PlayerImage,6);
  FreeImages(PShot1Image,2);
  FreeImages(PShot2Image,2);
  FreeImages(PShot3Image,3);

  FreeImages(EShotImage,4);
  FreeImages(ELaserImage,1);
  FreeImages(EMissileImage,8);
  FreeImages(EBoundImage,8);
  FreeImages(ERingImage,4);

  FreeImages(BombImage,5);
  FreeImages(LargeBombImage,5);

  FreeImages(Enemy1Image,8);
  FreeImages(Enemy2Image,8);
  FreeImages(Enemy3Image,8);
  FreeImages(Enemy4Image,8);
  FreeImages(Enemy5Image,4);
  FreeImages(Enemy6Image,6);
  FreeImages(Enemy7Image,1);

  FreeImages(Boss1Image,1);
  FreeImages(Boss2Image,1);
  FreeImages(Boss3Image,1);
  FreeImages(Boss4Image,1);
  FreeImages(Boss5Image,1);
  FreeImages(Boss6Image,2);
  FreeImages(Boss7Image,1);

  FreeImages(ItemImage,4);

  DeleteAllStar();

  XFreeGC(dpy, FontGC);
  XFreeGC(dpy, BackGC);
  XFreeGC(dpy, FillGC);
  
  XAutoRepeatOn(dpy);
  XFlush(dpy);
  XCloseDisplay(dpy);
  return 0;
}


int draw_string(int x, int y, const char *string, int length)
{
  XDrawString(dpy, WorkPixmap, FontGC, x, y, string, length);
  return 0;
}



#if 0
/* return the width of the char c on success,
 * negative value on error */
int draw_char(int x, int y, int c)
{
  if (!isprint(c))
  {
    fprintf(stderr, "draw_char: unprintable char found (\\x%x), "
            "replacing it with ?\n", c);
    c = '?';
  }

  /* don't assume ASCII */
  switch (c)
  {
  case ' ':
    /* do nothing */
    return 4;
  case '!':
    XCopyArea(dpy, image_font, win, FontGC,
              0, 0, 4, 12, x, y);
    return 4;
  case '"':
    XCopyArea(dpy, image_font, win, FontGC,
              4, 0, 5, 12, x, y);
    return 5;
  case '#':
    XCopyArea(dpy, image_font, win, FontGC,
              9, 0, 7, 12, x, y);
    return 7;
  case '$':
    XCopyArea(dpy, image_font, win, FontGC,
              16, 0, 7, 12, x, y);
    return 7;
  case '%':
    XCopyArea(dpy, image_font, win, FontGC,
              23, 0, 10, 12, x, y);
    return 10;
  case '%':
    XCopyArea(dpy, image_font, win, FontGC,
              33, 0, 7, 12, x, y);
    return 7;



  default:
    fprintf(stderr, "draw_char: unknown char found (\\x%x), ignoring", c);
    return -1;
  }
    
    
  }
  /* should not reach here */
  return -2;
}

#endif /* 0 */
